import datetime
import unittest

from fsl import functions
import tutils

FN_NONEXISTING = './__nonexisting_filename__'
FN_FILEA = tutils.ROOTDIR + '/filea'
FN_HIDDEN = tutils.ROOTDIR + '/.hidden'

class TestFunctions(unittest.TestCase):
    
    def _generic_nonexisting_file(self, funcname):
        try:
            functions.execute_function(funcname, [FN_NONEXISTING])
            self.fail('Expected function %s to fail with non-existing file as parameter' % funcname)
        except (OSError, IOError):
            pass

    def _generic_success(self, funcname, params, exp_types, exp_result):
        if not isinstance(exp_types, list):
            exp_types = [exp_types]

        result = functions.execute_function(funcname, params)

        for exp_type in exp_types:
            if isinstance(result, exp_type):
                break
        else:
            self.failUnless(isinstance(result, exp_type),
                'Function "%s": expected result of type(s) %s, got %s' % (funcname, exp_types, type(result)))

        self.assertEqual(result, exp_result,
            'Function "%s" gave incorrect result: expected "%s", got "%s"' % (funcname, exp_result, result))

    # Invalid parameters to execute_function

    def test_invalid_function(self):
        try:
            # Function "turingmachinehalts" is not implemented
            # and probably never will be.
            functions.execute_function('turingmachinehalts', [])
            self.fail("funtions.execute_function didn't fail with an invalid function name")
        except ValueError:
            pass

    def test_invalid_param_format(self):
        try:
            functions.execute_function('age', 'abc')
            self.fail("funtions.execute_function didn't fail when giving non-list as function-parameters")
        except TypeError:
            pass

    def test_no_params(self):
        try:
            functions.execute_function('age')
            self.fail("funtions.execute_function didn't fail when giving no function-parameters")
        except TypeError:
            pass

    # age

    def test_age_success(self):
        result = functions.execute_function('age', [FN_FILEA])
        self.failUnless(isinstance(result, float),
            'Expected float result, got "%s"' % result)

    def test_age_failure(self):
        self._generic_nonexisting_file('age')

    # base

    # date

    def test_date_success(self):
        exp_datetime = datetime.datetime(2005, 5, 5, 15, 25, 0)
        self._generic_success('date', [FN_FILEA], [datetime.datetime], exp_datetime)

    def test_date_failure(self):
        self._generic_nonexisting_file('date')

    # exists

    def test_exists_success_true(self):
        self._generic_success('exists', [FN_FILEA], [bool], True)

    def test_exists_success_false(self):
        self._generic_success('exists', [FN_NONEXISTING], [bool], False)

    # extract

    def test_extract_success(self):
        # 2005-08-10 11:57:03 Wednesday (weekday=3), week=32
        dt = datetime.datetime(2005, 8, 10, 11, 57, 3)
        self._generic_success('extract', [dt, 'year'], [int, long], 2005)
        self._generic_success('extract', [dt, 'month'], [int, long], 8)
        self._generic_success('extract', [dt, 'day'], [int, long], 10)
        self._generic_success('extract', [dt, 'hour'], [int, long], 11)
        self._generic_success('extract', [dt, 'hours'], [int, long], 11)
        self._generic_success('extract', [dt, 'minute'], [int, long], 57)
        self._generic_success('extract', [dt, 'minutes'], [int, long], 57)
        self._generic_success('extract', [dt, 'second'], [int, long], 3)
        self._generic_success('extract', [dt, 'seconds'], [int, long], 3)
        self._generic_success('extract', [dt, 'weekday'], [int, long], 3)
        self._generic_success('extract', [dt, 'week'], [int, long], 32)

    def test_extract_failure_date(self):
        try:
            functions.execute_function('extract', ['2005-05-05 12:30', 'year'])
            self.fail('Function "extract" accepted string as datetime parameter')
        except TypeError:
            pass

    def test_extract_failure_part(self):
        dt = datetime.datetime(2005, 8, 10, 11, 57, 3)
        try:
            functions.execute_function('extract', [dt, 'xxx'])
            self.fail('Function "extract" accepted invalid part parameter')
        except ValueError:
            pass

    # now

    def test_now_success(self):
        realnow = datetime.datetime.now()
        result = functions.execute_function('now', [])

        self.failUnless(isinstance(result, datetime.datetime),
            'Expected datetime result, got "%s"' % result)

        hourdiff = abs(tutils.timedelta2hours(result - realnow))
        # Allow 5 seconds of error.
        self.failIf(hourdiff > 5.0/3600,
            'Function "now" returned incorrect time (difference: %.1f hours)' % hourdiff)

    # size

    def test_size_success(self):
        self._generic_success('size', [FN_FILEA], [int, long], 100)

    def test_size_success_zero(self):
        self._generic_success('size', [FN_HIDDEN], [int, long], 0)

    def test_size_failure(self):
        self._generic_nonexisting_file('size')


def main(args):
    unittest.main()

if __name__=='__main__':
    import sys
    main(sys.argv[1:])
